from ._stacify import stacify
from ._preview import (
    preview,
    preview_aux_data,
    preview_rgb,
)
from ._catalog import create_stac_catalog

__all__ = [
    _stacify,
    create_stac_catalog,
    _preview,
    preview_rgb,
    preview_aux_data,
]



import pystac
from pathlib import Path
import xarray as xr
import stackstac
import numpy as np


from s2stac._catalog import create_stac_catalog


def stacify(
    safe_folders: list[Path],
    catalog_folder: Path | None = None,
    # Include flags
    include_img_data: bool = True,
    include_qi_data: bool = True,
    include_aux_data: bool = True,
    include_footprint: bool = True,
    include_cloud_proba: bool = True,
    include_ecmwf: bool = True,
    include_mtd_tl: bool = True,
    # Stackstac parameters
    epsg: int | None = None,
    **kwargs,
) -> pystac.Catalog:
    catalog = create_stac_catalog(
        safe_folders=safe_folders,
        catalog_folder=catalog_folder,
        include_img_data=include_img_data,
        include_qi_data=include_qi_data,
        include_aux_data=include_aux_data,
        include_footprint=include_footprint,
        include_cloud_proba=include_cloud_proba,
        include_ecmwf=include_ecmwf,
        include_mtd_tl=include_mtd_tl,
    )
    items = list(catalog.get_all_items())

    if len(set(item.properties["proj:epsg"] for item in items)) > 1 and epsg is None:
        epsg = items[0].properties["proj:epsg"]
    stack = stackstac.stack(items, epsg=epsg, chunksize=(5000, 5000), **kwargs)

    # Turn no data (0) into NaNs
    stack = xr.where(stack, stack, np.nan)
    return stack



import pystac
from pystac.extensions.eo import Band
from pathlib import Path
import rioxarray

from ._asset import add_asset


def add_img_data_var(
    item: pystac.Item,
    img_data_var_path: Path,
    img_data_var_label: str,
    img_data_var_description: str,
    tmp_folder: Path,
) -> Band:
    img_data_var_da = rioxarray.open_rasterio(img_data_var_path)

    return add_asset(
        item=item,
        asset_path=img_data_var_path,
        asset_label=img_data_var_label,
        asset_description=img_data_var_description,
        asset_common_name=img_data_var_label,
        asset_nodata=0,
        asset_epsg=f"EPSG:{img_data_var_da.rio.crs.to_epsg()}",
        asset_geometry=item.geometry,
        asset_bbox=item.bbox,
        asset_shape=img_data_var_da.shape[-2:],
        asset_transform=list(img_data_var_da.rio.transform()),
    )


def add_img_data_bands(
    item: pystac.Item,
    img_data_folder: Path,
    tmp_folder: Path,
) -> list[Band]:
    return [
        add_img_data_var(
            item=item,
            img_data_var_path=list(
                img_data_folder.glob(f"*{img_data_res}*/*{img_data_label}*.jp2")
            )[0],
            img_data_var_label=img_data_label,
            img_data_var_description=img_data_description,
            tmp_folder=tmp_folder,
        )
        for img_data_res, img_data_label, img_data_description in [
            ("10", "AOT", ""),
            ("10", "B02", ""),
            ("10", "B03", ""),
            ("10", "B04", ""),
            ("10", "B08", ""),
            ("10", "TCI", ""),
            ("10", "WVP", ""),
            ("20", "B05", ""),
            ("20", "B06", ""),
            ("20", "B07", ""),
            ("20", "B8A", ""),
            ("20", "B11", ""),
            ("20", "B12", ""),
            ("20", "SCL", ""),
            ("60", "B01", ""),
            ("60", "B09", ""),
        ]
    ]



import pystac
from pystac.extensions.eo import Band
from pathlib import Path
import xarray as xr
import rasterio
import rasterio.features
import rioxarray
import numpy as np
import geopandas as gpd

from ._asset import add_asset


def rasterize_footprint(
    footprint_path: Path,
    target_da: xr.DataArray,
) -> xr.DataArray:
    gdf = gpd.read_file(footprint_path, driver="GML")
    return xr.DataArray(
        rasterio.features.rasterize(
            shapes=[
                (row["geometry"], int(row["gml_id"].split("-")[-2]))
                for _, row in gdf.iterrows()
            ],
            transform=target_da.rio.transform(),
            out_shape=target_da.shape[-2:],
            fill=0,
            dtype=np.uint8,
        ),
        coords={
            "y": target_da.y,
            "x": target_da.x,
        },
        dims=["y", "x"],
    ).rio.write_crs(target_da.rio.crs)


def add_footprint(
    item: pystac.Item,
    footprint_path: Path,
    footprint_label: str,
    footprint_description: str,
    target_da: xr.DataArray,
    tmp_folder: Path,
) -> Band:
    if footprint_path.suffix == ".gml":
        tmp_footprint_path = tmp_folder / footprint_path.with_suffix(".jp2").name
        if tmp_footprint_path.exists():
            footprint_da = rioxarray.open_rasterio(tmp_footprint_path)
        else:
            footprint_da = rasterize_footprint(
                footprint_path=footprint_path,
                target_da=target_da,
            )
            footprint_da.rio.to_raster(tmp_footprint_path)
            footprint_da = rioxarray.open_rasterio(tmp_footprint_path)
        footprint_path = tmp_footprint_path
    else:
        footprint_da = rioxarray.open_rasterio(footprint_path)

    return add_asset(
        item=item,
        asset_path=footprint_path,
        asset_label=footprint_label,
        asset_description=footprint_description,
        asset_common_name=footprint_label,
        asset_nodata=0,
        asset_epsg=f"EPSG:{footprint_da.rio.crs.to_epsg()}",
        asset_geometry=item.geometry,
        asset_bbox=item.bbox,
        asset_shape=footprint_da.shape[-2:],
        asset_transform=list(footprint_da.rio.transform()),
    )


def add_footprints(
    item: pystac.Item,
    qi_folder: Path,
    target_da_10m: xr.DataArray,
    target_da_20m: xr.DataArray,
    target_da_60m: xr.DataArray,
    tmp_folder: Path,
) -> list[Band]:
    return [
        add_footprint(
            item=item,
            footprint_path=(
                list(qi_folder.glob(f"*{footprint_label}*.jp2"))
                + list(qi_folder.glob(f"*{footprint_label}*.gml"))
            )[0],
            footprint_label=footprint_label,
            footprint_description=footprint_description,
            target_da=target_da,
            tmp_folder=tmp_folder,
        )
        for footprint_res, footprint_label, footprint_description, target_da in [
            ("60", "MSK_DETFOO_B01", "", target_da_60m),
            ("10", "MSK_DETFOO_B02", "", target_da_10m),
            ("10", "MSK_DETFOO_B03", "", target_da_10m),
            ("10", "MSK_DETFOO_B04", "", target_da_10m),
            ("20", "MSK_DETFOO_B05", "", target_da_20m),
            ("20", "MSK_DETFOO_B06", "", target_da_20m),
            ("20", "MSK_DETFOO_B07", "", target_da_20m),
            ("10", "MSK_DETFOO_B08", "", target_da_10m),
            ("20", "MSK_DETFOO_B8A", "", target_da_20m),
            ("60", "MSK_DETFOO_B09", "", target_da_60m),
            ("20", "MSK_DETFOO_B10", "", target_da_20m),
            ("20", "MSK_DETFOO_B11", "", target_da_20m),
            ("20", "MSK_DETFOO_B12", "", target_da_20m),
        ]
    ]


def add_msk_cldprb(
    item: pystac.Item,
    msk_cldprb_path: Path,
    msk_cldprb_label: str,
    msk_cldprb_description: str,
    tmp_folder: Path,
) -> Band:
    msk_cldprb_da = rioxarray.open_rasterio(msk_cldprb_path)

    return add_asset(
        item=item,
        asset_path=msk_cldprb_path,
        asset_label=msk_cldprb_label,
        asset_description=msk_cldprb_description,
        asset_common_name=msk_cldprb_label,
        asset_nodata=0,
        asset_epsg=f"EPSG:{msk_cldprb_da.rio.crs.to_epsg()}",
        asset_geometry=item.geometry,
        asset_bbox=item.bbox,
        asset_shape=msk_cldprb_da.shape[-2:],
        asset_transform=list(msk_cldprb_da.rio.transform()),
    )


def add_qi_data(
    item: pystac.Item,
    qi_folder: Path,
    target_da_10m: xr.DataArray,
    target_da_20m: xr.DataArray,
    target_da_60m: xr.DataArray,
    tmp_folder: Path,
    include_footprint: bool = True,
    include_cloud_proba: bool = True,
) -> list[Band]:
    bands = []
    if include_cloud_proba:
        bands += [
            add_msk_cldprb(
                item=item,
                msk_cldprb_path=qi_folder / "MSK_CLDPRB_20m.jp2",
                msk_cldprb_label="MSK_CLDPRB",
                msk_cldprb_description="Coud probability [0-255]",
                tmp_folder=tmp_folder,
            )
        ]
    if include_footprint:
        bands += add_footprints(
            item=item,
            qi_folder=qi_folder,
            target_da_10m=target_da_10m,
            target_da_20m=target_da_20m,
            target_da_60m=target_da_60m,
            tmp_folder=tmp_folder,
        )
    return bands



import pystac
from pystac.extensions.eo import Band
from pathlib import Path
import rioxarray
import xarray as xr

from ._asset import add_asset


def add_ecmwf_var(
    item: pystac.Item,
    ecmwft_var_label: str,
    ecmwft_var_da: xr.DataArray,
    target_da: xr.DataArray,
    tmp_folder: Path,
) -> Band:
    ecmwft_var_path = tmp_folder / f"{ecmwft_var_label}.tiff"
    if ecmwft_var_path.exists():
        ecmwft_var_da = rioxarray.open_rasterio(ecmwft_var_path)
    else:
        ecmwft_var_da = ecmwft_var_da.rename({"latitude": "y", "longitude": "x"})
        ecmwft_var_da = ecmwft_var_da.rio.write_crs("epsg:4326")
        ecmwft_var_da = ecmwft_var_da.transpose("y", "x")
        ecmwft_var_da = ecmwft_var_da.rio.write_nodata(
            ecmwft_var_da.attrs["GRIB_missingValue"]
        )
        ecmwft_var_da = ecmwft_var_da.rio.reproject(
            target_da.rio.crs,
            shape=ecmwft_var_da.shape,
        )
        ecmwft_var_da.rio.to_raster(ecmwft_var_path)

    return add_asset(
        item=item,
        asset_path=ecmwft_var_path,
        asset_label=ecmwft_var_label,
        asset_description=ecmwft_var_label,
        asset_common_name=ecmwft_var_label,
        asset_nodata=0,
        asset_epsg=f"EPSG:{ecmwft_var_da.rio.crs.to_epsg()}",
        asset_geometry=item.geometry,
        asset_bbox=item.bbox,
        asset_shape=ecmwft_var_da.shape[-2:],
        asset_transform=list(ecmwft_var_da.rio.transform()),
    )


def add_ecmwf(
    item: pystac.Item,
    ecmwft_path: Path,
    target_da: xr.DataArray,
    tmp_folder: Path,
) -> list[Band]:
    tmp_folder = tmp_folder / "ecmwft"
    tmp_folder.mkdir(exist_ok=True)

    ecmwft_da = xr.open_dataset(ecmwft_path, engine="cfgrib")
    return [
        add_ecmwf_var(
            item=item,
            ecmwft_var_label=ecmwft_var_label,
            ecmwft_var_da=ecmwft_da[ecmwft_var_label],
            target_da=target_da,
            tmp_folder=tmp_folder,
        )
        for ecmwft_var_label in ecmwft_da.data_vars
        if ecmwft_var_label in ["msl", "tco3", "tcwv"]
    ]



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['s2stac']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.12.2,<5.0.0',
 'bokeh==2.4.2',
 'cfgrib>=0.9.10.3,<0.10.0.0',
 'dask>=2023.5.0,<2024.0.0',
 'distributed>=2023.5.0,<2024.0.0',
 'eccodes==1.2.0',
 'geopandas>=0.13.0,<0.14.0',
 'jinja2==3.0.3',
 'lxml>=4.9.2,<5.0.0',
 'matplotlib>=3.7.1,<4.0.0',
 'numpy>=1.24.3,<2.0.0',
 'pygeoapi>=0.14.0,<0.15.0',
 'pygrib>=2.1.4,<3.0.0',
 'pystac-client>=0.6.1,<0.7.0',
 'pystac>=1.7.3,<2.0.0',
 'rasterio>=1.3.6,<2.0.0',
 'rioxarray>=0.14.1,<0.15.0',
 'shapely==1.8.5',
 'stackstac>=0.4.3,<0.5.0',
 'xarray>=2023.4.2,<2024.0.0']

setup_kwargs = {
    'name': 's2stac',
    'version': '0.1.1',
    'description': 'Create a STAC from local S2 data.',
    'long_description': '# s2stac\n\nEasily access many Sentinel-2 safe folders from a convenient xarray Dataset.\n\n## Purpose\n\nThe purpose of this library is to create on-the-fly STAC from a list Sentinel-2 products in the form of SAFE folders and provide a xarray Dataset to easily access the main Sentinel-2 bands and more!\n\nAccessible information includes:\n- all bands from the IMG_DATA folder (B01, B02, ..., B11)\n- all viewing/sun zenith/azimuth angles.\n- all detector footprints (for each band)\n- ECMWF data (msl, tco3, ...)\n\n## How it works\n\ns2stac uses the stackstac library internally to interogate a static STAC database created on the fly with pystac.\nThe database can be persisted so the user does not need to build it everytime.\n\n## Installation\n\nThis library requires Python 3.10 or higher.\n\nThis library also requires the eccodes library (version 2.16.0) to be able to parse the GRIB ECMWF dataset containing the meteorological data associated with each tile.\nYou can install it on Ubuntu with the following command:\n```bash\nsudo apt-get install libeccodes-dev\n```\n\nNow you can simply install s2stac with:\n```bash\npip install s2stac\n```\n\n## Usage\n\n```python\nfrom pathlib import Path\nfrom s2stac import stacify\n\nsafe_folders = list(Path("/path/to/your/safe/folders").glob("*.SAFE"))\n\nstack = stacify(safe_folders)\nstack\n```\n\nExemple of output:\n```bash\n<xarray.DataArray \'stackstac-67b9dd4d3bf2316dde72ffa86206dab6\' (time: 1,\n                                                                band: 59,\n                                                                y: 12352,\n                                                                x: 12352)>\ndask.array<where, shape=(1, 59, 12352, 12352), dtype=float64, chunksize=(1, 1, 5000, 5000), chunktype=numpy.ndarray>\nCoordinates: (12/16)\n  * time                 (time) datetime64[ns] 2023-06-02T15:00:42.812526\n    id                   (time) <U60 \'S2B_MSIL2A_20190222T095029_N0211_R079_T...\n  * band                 (band) <U16 \'AOT\' \'B02\' ... \'VIEW_ZENITH_B12\'\n  * x                    (x) float64 4.931e+05 4.931e+05 ... 6.166e+05 6.166e+05\n  * y                    (y) float64 6.709e+06 6.709e+06 ... 6.585e+06 6.585e+06\n    gsd                  int64 10\n    ...                   ...\n    raster:bands         object {\'nodata\': 0}\n    proj:geometry        object {\'type\': \'Polygon\', \'coordinates\': (((499980....\n    common_name          (band) <U16 \'AOT\' \'B02\' ... \'VIEW_ZENITH_B12\'\n    center_wavelength    object None\n    full_width_half_max  object None\n    epsg                 <U10 \'EPSG:32634\'\n```\n\n## License\n\nMIT\n\n\n## Author\n\nPierre Louvart - plouvart@argans.eu\n\n## Credit\n\ngjoseph92, the creator of [https://stackstac.readthedocs.io/en/latest/](stackstac)\n\n',
    'author': 'Pierre Louvart',
    'author_email': 'pierre.louvart@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10.5,<4.0.0',
}


setup(**setup_kwargs)



import shutil
from tempfile import TemporaryDirectory
import pystac
from pystac.extensions.eo import EOExtension
from pystac.extensions.projection import ProjectionExtension
from pathlib import Path
import rioxarray

from datetime import datetime, timezone
from shapely.geometry import Polygon, mapping
from ._aux_data import add_ecmwf

from ._img_data import add_img_data_bands
from ._mtd_tl import add_mtd_tl
from ._qi_data import add_qi_data


class __TmpFolderPersistence:
    def __init__(self):
        self.tmp_folders = []

    def add_tmp_folder(
        self,
    ) -> Path:
        tmp_folder = TemporaryDirectory()
        self.tmp_folders.append(tmp_folder)
        return Path(tmp_folder.name)

    def __del__(self):
        for tmp_folder in self.tmp_folders:
            tmp_folder.cleanup()


__tmp_folder_persistence = __TmpFolderPersistence()


def safe_folder_to_item(
    safe_folder: Path,
    catalog_folder: Path,
    include_img_data: bool = True,
    include_aux_data: bool = True,
    include_qi_data: bool = True,
    include_footprint: bool = True,
    include_cloud_proba: bool = True,
    include_ecmwf: bool = True,
    include_mtd_tl: bool = True,
) -> pystac.Item:
    target_da_10m = rioxarray.open_rasterio(
        list(safe_folder.glob("GRANULE/*/IMG_DATA/*10*/*B02*"))[0]
    )
    target_da_20m = rioxarray.open_rasterio(
        list(safe_folder.glob("GRANULE/*/IMG_DATA/*20*/*B02*"))[0]
    )
    target_da_60m = rioxarray.open_rasterio(
        list(safe_folder.glob("GRANULE/*/IMG_DATA/*60*/*B02*"))[0]
    )

    catalog_folder = catalog_folder / safe_folder.stem
    catalog_folder.mkdir(exist_ok=True)

    bbox = target_da_10m.rio.bounds()
    xmin, ymin, xmax, ymax = bbox
    footprint = mapping(
        Polygon(
            [
                [xmin, ymin],
                [xmin, ymax],
                [xmax, ymax],
                [xmax, ymin],
            ]
        )
    )
    datetime_utc = datetime.now(tz=timezone.utc)

    # Item definition
    item_id = safe_folder.stem
    item = pystac.Item(
        id=item_id,
        geometry=footprint,
        bbox=bbox,
        datetime=datetime_utc,
        properties={},
    )

    item.common_metadata.platform = "MSI"
    item.common_metadata.instruments = ["Sentinel-2"]
    item.common_metadata.gsd = 10

    # Add EO data
    eo_extension = EOExtension.ext(item, add_if_missing=True)

    bands = []
    # Write MSK_DETFOO data
    if include_img_data:
        bands += add_img_data_bands(
            item=item,
            img_data_folder=list(safe_folder.glob("GRANULE/*/IMG_DATA/"))[0],
            tmp_folder=catalog_folder,
        )
    if include_qi_data:
        bands += add_qi_data(
            item=item,
            qi_folder=list(safe_folder.glob("GRANULE/*/QI_DATA/"))[0],
            target_da_10m=target_da_10m,
            target_da_20m=target_da_20m,
            target_da_60m=target_da_60m,
            tmp_folder=catalog_folder,
            include_footprint=include_footprint,
            include_cloud_proba=include_cloud_proba,
        )
    if include_aux_data:
        bands += add_ecmwf(
            item=item,
            ecmwft_path=list(safe_folder.glob("GRANULE/*/AUX_DATA/AUX_ECMWFT"))[0],
            target_da=target_da_10m,
            tmp_folder=catalog_folder,
        )
    if include_mtd_tl:
        bands += add_mtd_tl(
            item=item,
            mtd_tl_path=list(safe_folder.glob("GRANULE/*/MTD_TL.xml"))[0],
            target_da=target_da_10m,
            tmp_folder=catalog_folder,
        )

    eo_extension.apply(
        bands=bands,
    )

    # Add Projection
    projection_extension = ProjectionExtension.ext(item, add_if_missing=True)
    projection_extension.apply(
        epsg=target_da_10m.rio.crs.to_epsg(),
        transform=list(target_da_10m.rio.transform()),
        shape=target_da_10m.shape[-2:],
        bbox=bbox,
    )

    return item


def create_stac_catalog(
    safe_folders: list[Path],
    catalog_folder: Path | None = None,
    overwrite_catalog_folder: bool = False,
    include_img_data: bool = True,
    include_qi_data: bool = True,
    include_aux_data: bool = True,
    include_footprint: bool = True,
    include_cloud_proba: bool = True,
    include_ecmwf: bool = True,
    include_mtd_tl: bool = True,
) -> pystac.Catalog:
    if catalog_folder is None:
        catalog_folder = __tmp_folder_persistence.add_tmp_folder()
    elif overwrite_catalog_folder:
        shutil.rmtree(path=str(catalog_folder))
    catalog_folder.mkdir(exist_ok=True, parents=True)

    catalog = pystac.Catalog(id="s2-catalog", description="Sentinel-2 STAC")

    for safe_folder in safe_folders:
        catalog.add_item(
            safe_folder_to_item(
                safe_folder=safe_folder,
                catalog_folder=catalog_folder,
                include_img_data=include_img_data,
                include_aux_data=include_aux_data,
                include_qi_data=include_qi_data,
                include_footprint=include_footprint,
                include_cloud_proba=include_cloud_proba,
                include_ecmwf=include_ecmwf,
                include_mtd_tl=include_mtd_tl,
            )
        )

    catalog.normalize_hrefs(str(catalog_folder))
    catalog.save(catalog_type=pystac.CatalogType.RELATIVE_PUBLISHED)

    return catalog



import xarray as xr
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable


def preview_rgb(
    ds: xr.Dataset,
    min_percentile: float = 5.0,
    max_percentile: float = 95.0,
    maximal_length: int = 200,
) -> None:
    w, h = ds.shape[-2:]

    step, mod = divmod(w if w > h else h, maximal_length)
    step -= mod != 0

    img = (
        ds.loc[:, :, ::step, ::step]
        .sel(
            band=[
                "B04",
                "B03",
                "B02",
            ]
        )
        .median("time")
        .compute()
        / 10_000
    )
    min_quantile, max_quantile = min_percentile / 100, max_percentile / 100
    quantiles = img.quantile([min_quantile, max_quantile], dim=("x", "y"))
    quantiles["quantile"] = ["min", "max"]
    min_quantiles = quantiles.sel(quantile="min")
    max_quantiles = quantiles.sel(quantile="max")

    normalized_img = (img - min_quantiles) / (max_quantiles - min_quantiles)

    clipped_img = normalized_img.clip(min=0, max=1)
    clipped_img.plot.imshow()
    plt.gca().set_aspect(aspect="equal")
    plt.show()


def preview_aux_data(
    ds: xr.Dataset,
    maximal_length: int = 200,
) -> None:
    w, h = ds.shape[-2:]

    step, mod = divmod(w if w > h else h, maximal_length)
    step -= mod != 0

    img = (
        ds.loc[:, :, ::step, ::step]
        .sel(
            band=["msl", "tco3", "tcwv"],
        )
        .median("time")
        .compute()
    )

    fig, axes = plt.subplots(1, 3)
    for ax, band_name in zip(axes, img.band):
        im = img.sel(band=band_name).plot.imshow(ax=ax, add_colorbar=False)
        ax.set_aspect(aspect="equal")
        divider = make_axes_locatable(ax)
        cax = divider.append_axes("right", size="5%", pad=0.05)
        plt.colorbar(im, cax=cax)
    plt.show()


def preview(
    ds: xr.Dataset,
    maximal_length: int = 200,
) -> None:
    if set(["B02", "B03", "B04"]).issubset(set(ds.band.values)):
        preview_rgb(ds, maximal_length=maximal_length)
    elif set(["msl", "tco3", "tcwv"]).issubset(set(ds.band.values)):
        preview_aux_data(ds, maximal_length=maximal_length)



from typing import Any
import pystac
from pystac.extensions.eo import Band, EOExtension
from pystac.extensions.projection import ProjectionExtension
from pystac.extensions.raster import RasterBand, RasterExtension
from pathlib import Path


def add_asset(
    item: pystac.Item,
    asset_path: Path,
    asset_label: str,
    asset_description: str,
    asset_common_name: str,
    asset_nodata: float,
    asset_epsg: str,
    asset_geometry: dict[str, Any],
    asset_bbox: list[float],
    asset_shape: list[int],
    asset_transform: list[float],
) -> Band:
    asset = pystac.Asset(href=str(asset_path), media_type=pystac.MediaType.JPEG2000)
    # Add image to asset
    item.add_asset(asset_label, asset)
    # Add Band to asset
    asset_band = Band.create(
        name=asset_label,
        description=asset_description,
        common_name=asset_common_name,
    )
    eo_on_asset = EOExtension.ext(item.assets[asset_label])
    eo_on_asset.apply(bands=[asset_band])
    # Add Raster Band to asset
    asset_raster_band = RasterBand.create(nodata=asset_nodata)
    raster_on_asset = RasterExtension.ext(item.assets[asset_label], add_if_missing=True)
    raster_on_asset.apply(bands=[asset_raster_band])
    # Add Projection to asset
    proj_on_asset = ProjectionExtension.ext(
        item.assets[asset_label], add_if_missing=True
    )
    proj_on_asset.apply(
        epsg=asset_epsg,
        geometry=asset_geometry,
        bbox=asset_bbox,
        shape=asset_shape,
        transform=asset_transform,
    )
    return asset_band



import pystac
from pystac.extensions.eo import Band
from pathlib import Path
import xarray as xr
import rioxarray
import numpy as np
from bs4 import BeautifulSoup
from bs4.element import Tag

from ._asset import add_asset


def parse_xml_array(tag: Tag) -> np.array:
    return np.array(
        [list(map(float, i.string.split())) for i in tag.Values_List.find_all("VALUES")]
    )


def mtd_tl_to_dataset(
    mtd_tl_path: Path,
    target_da: xr.DataArray,
) -> xr.Dataset:
    soup = BeautifulSoup(open(mtd_tl_path, "r").read(), features="xml")

    xmin = float(
        soup.Tile_Geocoding.findAll("Geoposition", {"resolution": "10"})[0].ULX.string
    )
    xmax = xmin + float(
        soup.Tile_Geocoding.findAll("Size", {"resolution": "10"})[0].NCOLS.string
    ) * float(
        soup.Tile_Geocoding.findAll("Geoposition", {"resolution": "10"})[0].XDIM.string
    )

    ymin = float(
        soup.Tile_Geocoding.findAll("Geoposition", {"resolution": "10"})[0].ULY.string
    )
    ymax = ymin + float(
        soup.Tile_Geocoding.findAll("Size", {"resolution": "10"})[0].NROWS.string
    ) * float(
        soup.Tile_Geocoding.findAll("Geoposition", {"resolution": "10"})[0].YDIM.string
    )

    sun_zenith_data = parse_xml_array(soup.Tile_Angles.Sun_Angles_Grid.Zenith)
    sun_azimuth_data = parse_xml_array(soup.Tile_Angles.Sun_Angles_Grid.Azimuth)

    nb_cols, nb_rows = sun_zenith_data.shape
    # Should we correct the offset or not?
    xoffset = 0  # (xmax - xmin) / (nb_cols + 1)
    yoffset = 0  # (ymax - ymin) / (nb_rows + 1)
    x = np.linspace(xmin + xoffset, xmax - xoffset, nb_cols)
    y = np.linspace(ymin + yoffset, ymax - yoffset, nb_rows)

    sun_zenith_array = xr.DataArray(
        sun_zenith_data, coords={"y": y, "x": x}, dims=["y", "x"]
    )
    sun_azimuth_array = xr.DataArray(
        sun_azimuth_data, coords={"y": y, "x": x}, dims=["y", "x"]
    )

    mean_sun_zenith = float(soup.Mean_Sun_Angle.ZENITH_ANGLE.string)
    mean_sun_azimuth = float(soup.Mean_Sun_Angle.AZIMUTH_ANGLE.string)

    bands_data = dict(
        sorted(
            {
                (
                    int(grid["bandId"]),
                    grid["detectorId"],
                ): (
                    parse_xml_array(grid.Zenith),
                    parse_xml_array(grid.Azimuth),
                )
                for grid in soup.Tile_Angles.find_all("Viewing_Incidence_Angles_Grids")
            }.items()
        )
    )
    band_ids = list(sorted(set([k[0] for k in bands_data.keys()])))
    detector_ids = list(sorted(set([k[1] for k in bands_data.keys()])))
    band_labels = list(map(lambda s: f"B{s:02d}", band_ids))

    bands_zenith_array = xr.DataArray(
        np.stack([[bands_data[(j, i)][0] for i in detector_ids] for j in band_ids]),
        coords={
            "band": band_labels,
            "detector": detector_ids,
            "y": y,
            "x": x,
        },
        dims=["band", "detector", "y", "x"],
    )
    bands_azimuth_array = xr.DataArray(
        np.stack([[bands_data[(j, i)][1] for i in detector_ids] for j in band_ids]),
        coords={
            "band": band_labels,
            "detector": detector_ids,
            "y": y,
            "x": x,
        },
        dims=["band", "detector", "y", "x"],
    )

    bands_mean_data = dict(
        sorted(
            {
                int(mean_viewing_angles["bandId"]): (
                    float(mean_viewing_angles.ZENITH_ANGLE.string),
                    float(mean_viewing_angles.AZIMUTH_ANGLE.string),
                )
                for mean_viewing_angles in (
                    soup.Mean_Viewing_Incidence_Angle_List.find_all(
                        "Mean_Viewing_Incidence_Angle"
                    )
                )
            }.items()
        )
    )
    bands_mean_zenith_array = xr.DataArray(
        [v[0] for v in bands_mean_data.values()],
        coords={
            "band": band_labels,
        },
        dims=["band"],
    )
    bands_mean_azimuth_array = xr.DataArray(
        [v[1] for v in bands_mean_data.values()],
        coords={
            "band": band_labels,
        },
        dims=["band"],
    )

    return xr.Dataset(
        {
            "view_mean_zenith": bands_mean_zenith_array,
            "view_mean_azimuth": bands_mean_azimuth_array,
            "view_zenith": bands_zenith_array,
            "view_azimuth": bands_azimuth_array,
            "sun_mean_zenith": mean_sun_zenith,
            "sun_mean_azimuth": mean_sun_azimuth,
            "sun_zenith": sun_zenith_array,
            "sun_azimuth": sun_azimuth_array,
        }
    ).rio.write_crs(target_da.rio.crs)


def add_viewing_azimuth_band(
    item: pystac.Item,
    viewing_azimuth_band_label: str,
    viewing_azimuth_band_da: xr.DataArray,
    tmp_folder: Path,
) -> Band:
    viewing_azimuth_band_path = tmp_folder / f"{viewing_azimuth_band_label}.tiff"
    if viewing_azimuth_band_path.exists():
        viewing_azimuth_band_da = rioxarray.open_rasterio(viewing_azimuth_band_path)
    else:
        viewing_azimuth_band_da.rio.to_raster(viewing_azimuth_band_path)

    return add_asset(
        item=item,
        asset_path=viewing_azimuth_band_path,
        asset_label=viewing_azimuth_band_label,
        asset_description=viewing_azimuth_band_label,
        asset_common_name=viewing_azimuth_band_label,
        asset_nodata=0,
        asset_epsg=f"EPSG:{viewing_azimuth_band_da.rio.crs.to_epsg()}",
        asset_geometry=item.geometry,
        asset_bbox=item.bbox,
        asset_shape=viewing_azimuth_band_da.shape[-2:],
        asset_transform=list(viewing_azimuth_band_da.rio.transform()),
    )


def add_viewing_zenith_band(
    item: pystac.Item,
    viewing_zenith_band_label: str,
    viewing_zenith_band_da: xr.DataArray,
    tmp_folder: Path,
) -> Band:
    viewing_zenith_band_path = tmp_folder / f"{viewing_zenith_band_label}.tiff"
    if viewing_zenith_band_path.exists():
        viewing_zenith_band_da = rioxarray.open_rasterio(viewing_zenith_band_path)
    else:
        viewing_zenith_band_da.rio.to_raster(viewing_zenith_band_path)

    return add_asset(
        item=item,
        asset_path=viewing_zenith_band_path,
        asset_label=viewing_zenith_band_label,
        asset_description=viewing_zenith_band_label,
        asset_common_name=viewing_zenith_band_label,
        asset_nodata=0,
        asset_epsg=f"EPSG:{viewing_zenith_band_da.rio.crs.to_epsg()}",
        asset_geometry=item.geometry,
        asset_bbox=item.bbox,
        asset_shape=viewing_zenith_band_da.shape[-2:],
        asset_transform=list(viewing_zenith_band_da.rio.transform()),
    )


def add_mtd_tl(
    item: pystac.Item,
    mtd_tl_path: Path,
    target_da: xr.DataArray,
    tmp_folder: Path,
) -> list[Band]:
    tmp_folder = tmp_folder / "mtd_tl"
    tmp_folder_viewing_zenith = tmp_folder / "viewing_zenith"
    tmp_folder_viewing_azimuth = tmp_folder / "viewing_azimuth"
    tmp_folder_viewing_zenith.mkdir(exist_ok=True, parents=True)
    tmp_folder_viewing_azimuth.mkdir(exist_ok=True, parents=True)
    mtd_tl_ds = mtd_tl_to_dataset(mtd_tl_path=mtd_tl_path, target_da=target_da)

    view_azimuth_da = mtd_tl_ds["view_azimuth"].mean("detector")
    view_zenith_da = mtd_tl_ds["view_zenith"].mean("detector")
    return [
        add_viewing_azimuth_band(
            item=item,
            tmp_folder=tmp_folder_viewing_azimuth,
            viewing_azimuth_band_da=view_azimuth_da.sel(band=view_azimuth_band_label),
            viewing_azimuth_band_label=f"VIEW_AZIMUTH_{view_azimuth_band_label.item()}",
        )
        for view_azimuth_band_label in view_azimuth_da.band
    ] + [
        add_viewing_zenith_band(
            item=item,
            tmp_folder=tmp_folder_viewing_zenith,
            viewing_zenith_band_da=view_zenith_da.sel(band=view_zenith_band_label),
            viewing_zenith_band_label=f"VIEW_ZENITH_{view_zenith_band_label.item()}",
        )
        for view_zenith_band_label in view_zenith_da.band
    ]
